@@ -74,6 +74,9 @@
 #'                \item{\code{lambda_l2}: L2 regularization (default = 0)}
 #'                \item{\code{lambda_l1}: L1 regularization (default = 0)}
 #'                \item{\code{max_bin}: Maximal number of bins that feature values will be bucketed in (default = 255)}
+#'                \item{\code{reuse_learning_rates_gp_model}: If TRUE, the learning rates for the covariance and potential 
+#'                auxiliary parameters are kept at the values from the previous boosting iteration and 
+#'                not re-initialized when optimizing them. Applies only to Gaussian process boosting (GPBoost algorithm) (default = FALSE)}
 #'                \item{\code{train_gp_model_cov_pars}: If TRUE, the covariance parameters of the Gaussian process 
 #'                are stimated in every boosting iterations, 
 #'                otherwise the gp_model parameters are not estimated. In the latter case, you need to 
@@ -83,7 +86,7 @@
 #'                (in addition to the tree model) for calculating predictions on the validation data 
 #'                (default = TRUE)}
 #'                \item{\code{leaves_newton_update}: Set this to TRUE to do a Newton update step for the tree leaves 
-#'                after the gradient step. Applies only to Gaussian process boosting (GPBoost algorithm)}
+#'                after the gradient step. Applies only to Gaussian process boosting (GPBoost algorithm) (default = FALSE)}
 #'                \item{num_threads: Number of threads. For the best speed, set this to
 #'                             the number of real CPU cores(\code{parallel::detectCores(logical = FALSE)}),
 #'                             not the number of threads (most CPU using hyper-threading to generate 2 threads
@@ -100,6 +103,9 @@
 #' boosting iterations, otherwise the \code{gp_model} parameters are not estimated. 
 #' In the latter case, you need to either estimate them beforehand or provide the values via 
 #' the \code{init_cov_pars} parameter when creating the \code{gp_model}
+#' @param reuse_learning_rates_gp_model Boolean. If TRUE, the learning rates for the covariance and potential 
+#'                auxiliary parameters are kept at the values from the previous boosting iteration and 
+#'                not re-initialized when optimizing them. Applies only to Gaussian process boosting (GPBoost algorithm)
 #' @section Early Stopping:
 #'
 #'          "early stopping" refers to stopping the training process if the model's performance on a given
@@ -208,6 +214,7 @@ gpboost <- function(data,
                     params = list(),
                     nrounds = 100L,
                     gp_model = NULL,
+                    reuse_learning_rates_gp_model = FALSE,
                     use_gp_model_for_validation = TRUE,
                     train_gp_model_cov_pars = TRUE,
                     valids = list(),
@@ -243,6 +250,7 @@ gpboost <- function(data,
     , "gp_model" = gp_model
     , "use_gp_model_for_validation" = use_gp_model_for_validation
     , "train_gp_model_cov_pars" = train_gp_model_cov_pars
+    , "reuse_learning_rates_gp_model" = reuse_learning_rates_gp_model
     , "valids" = valids
     , "obj" = obj
     , "eval" = eval
