@@ -189,10 +189,12 @@ namespace GPBoost {
 		*		For the GPBoost algorithm for non-Gaussian data, this is ignored (and can be nullptr) as the response data has been set before.
 		* \param fixed_effects Fixed effects component F of location parameter (only used for non-Gaussian data). For Gaussian data, this is ignored
 		* \param called_in_GPBoost_algorithm If true, this function is called in the GPBoost algorithm, otherwise for the estimation of a GLMM
+		* \param reuse_learning_rates_from_previous_call If true, the learning rates for the covariance and potential auxiliary parameters are kept at the values from the previous call and not re-initialized (can only be set to true if called_in_GPBoost_algorithm is true)
 		*/
 		void OptimCovPar(const double* y_data,
 			const double* fixed_effects,
-			bool called_in_GPBoost_algorithm);
+			bool called_in_GPBoost_algorithm,
+			bool reuse_learning_rates_from_previous_call);
 
 		/*!
 		* \brief Find covariance parameters and linear regression coefficients (if there are any) that minimize the (approximate) negative log-ligelihood
@@ -200,10 +202,12 @@ namespace GPBoost {
 		* \param y_data Response variable data
 		* \param covariate_data Covariate data (=independent variables, features)
 		* \param num_covariates Number of covariates
+		* \param fixed_effects Additional fixed effects that are added to the linear predictor (= offset)
 		*/
 		void OptimLinRegrCoefCovPar(const double* y_data,
 			const double* covariate_data,
-			int num_covariates);
+			int num_covariates,
+			const double* fixed_effects);
 
 		/*!
 		* \brief Find constant initial value of ensenmble for boosting (used only for non-Gaussian data). 
@@ -390,8 +394,10 @@ namespace GPBoost {
 		/*!
 		* \brief If cov_pars_ is is not defined, define them as init_cov_pars_ or if init_cov_pars_ is not given, find "reasonable" default values for the intial values of the covariance parameters
 		* \param y_data Response variable data used for finding intial values if cov_pars_ is not defined
+		* \param fixed_effects Additional fixed effects that are added to the linear predictor (= offset)
 		*/
-		void InitializeCovParsIfNotDefined(const double* y_data);
+		void InitializeCovParsIfNotDefined(const double* y_data,
+			const double* fixed_effects);
 
 		/*!
 		* \brief Return number of additional likelihood parameters (aux_pars_)
