@@ -171,6 +171,10 @@ namespace LightGBM {
 		// desc = **Note**: setting this to ``true`` may slow down the training
 		bool deterministic = false;
 
+		// desc = if ``true``, the learning rates for the covariance and potential auxiliary parameters are kept at the values from the previous boosting iteration and not re-initialized when optimizing them
+		// desc = applies only to the GPBoost algorithm
+		bool reuse_learning_rates_gp_model = false;
+
 		// desc = if ``true``, the covariance parameters of the Gaussian process / random effects model are trained (estimated) in every boosting iteration of the GPBoost algorithm, otherwise not
 		bool train_gp_model_cov_pars = true;
 
