@@ -2,10 +2,16 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
   
   context("GPModel_grouped_random_effects")
   
-  TOLERANCE <- 1E-3
-  TOL_LOOSE <- 1E-2
+  TOLERANCE_STRICT <- 1E-6
+  TOLERANCE_MEDIUM <- 1E-3
+  TOLERANCE_LOOSE <- 1E-2
   TOL_VERY_LOOSE <- 1E-1
-  TOL_STRICT <- 1E-6
+  DEFAULT_OPTIM_PARAMS <- list(optimizer_cov = "gradient_descent",
+                               lr_cov = 0.1, use_nesterov_acc = TRUE,
+                               acc_rate_cov = 0.5, delta_rel_conv = 1E-6,
+                               optimizer_coef = "gradient_descent", lr_coef = 0.1,
+                               convergence_criterion = "relative_change_in_log_likelihood")
+  DEFAULT_OPTIM_PARAMS_STD <- c(DEFAULT_OPTIM_PARAMS, list(std_dev = TRUE))
   
   # Function that simulates uniform random variables
   sim_rand_unif <- function(n, init_c=0.1){
@@ -50,17 +56,17 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     fit(gp_model, y = y, params = list(std_dev = TRUE, optimizer_cov = "fisher_scoring",
                                        convergence_criterion = "relative_change_in_parameters"))
     cov_pars <- c(0.49348532, 0.02326312, 1.22299521, 0.17995161)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
     expect_equal(dim(gp_model$get_cov_pars())[2], 2)
     expect_equal(dim(gp_model$get_cov_pars())[1], 2)
-    expect_equal(gp_model$get_num_optim_iter(), 6)
+    expect_equal(gp_model$get_num_optim_iter(), 5)
     # Can switch between likelihoods
     gp_model <- GPModel(group_data = group)
     gp_model$set_likelihood("gamma")
     gp_model$set_likelihood("gaussian")
     fit(gp_model, y = y, params = list(std_dev = TRUE, optimizer_cov = "fisher_scoring",
                                        convergence_criterion = "relative_change_in_parameters"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
     # Using gradient descent instead of Fisher scoring
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
@@ -70,7 +76,6 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expect_lt(sum(abs(cov_pars_est-cov_pars[c(1,3)])),1E-5)
     expect_equal(class(cov_pars_est), "numeric")
     expect_equal(length(cov_pars_est), 2)
-    expect_equal(gp_model$get_num_optim_iter(), 8)
     # Using gradient descent with Nesterov acceleration
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
@@ -78,29 +83,26 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          acc_rate_cov = 0.1, maxit = 1000,
                                          convergence_criterion = "relative_change_in_parameters"))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),1E-5)
-    expect_equal(gp_model$get_num_optim_iter(), 9)
     # Using gradient descent and a too large learning rate
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
                                          lr_cov = 10, use_nesterov_acc = FALSE,
                                          maxit = 1000, convergence_criterion = "relative_change_in_parameters"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 8)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOLERANCE_STRICT)
     # Different termination criterion
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_log_likelihood"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 5)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
     # Nelder-Mead
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "nelder_mead", std_dev = TRUE, delta_rel_conv=1e-6))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_MEDIUM)
     expect_equal(gp_model$get_num_optim_iter(), 12)
     # Adam
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "adam", std_dev = TRUE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_MEDIUM)
     expect_equal(gp_model$get_num_optim_iter(), 275)
     
     # Evaluation of likelihood
@@ -116,13 +118,13 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(-0.1553877, -0.3945731, 0)
     expected_cov <- c(0.5483871, 0.0000000, 0.0000000, 0.0000000,
                       0.5483871, 0.0000000, 0.0000000, 0.0000000, 2)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
     # Predict variances
     pred <- predict(gp_model, y=y, group_data_pred = group_test,
                     cov_pars = c(0.5,1.5), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE_STRICT)
     
     # Prediction from fitted model
     gp_model <- fitGPModel(group_data = group, y = y,
@@ -133,8 +135,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(-0.1543396, -0.3919117, 0.0000000)
     expected_cov <- c(0.5409198 , 0.0000000000, 0.0000000000, 0.0000000000,
                       0.5409198 , 0.0000000000, 0.0000000000, 0.0000000000, 1.7164805)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
     ## Cannot provide X for prediction when not provided for estimation
     expect_error(pred <- predict(gp_model, group_data_pred = group_test, X = group_test))
     
@@ -146,8 +148,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_unique <- unique(group)
     pred_random_effects <- predict(gp_model, group_data_pred = group_unique, 
                                    predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(training_data_random_effects[,1] - pred_random_effects$mu)),TOL_STRICT)
-    expect_lt(sum(abs(training_data_random_effects[,2] - pred_random_effects$var)),TOL_STRICT)
+    expect_lt(sum(abs(training_data_random_effects[,1] - pred_random_effects$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(training_data_random_effects[,2] - pred_random_effects$var)),TOLERANCE_STRICT)
     
     # Evaluate negative log-likelihood
     nll <- gp_model$neg_log_likelihood(cov_pars=c(0.1,1), y=y)
@@ -162,7 +164,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     # Do optimization using optim and e.g. Nelder-Mead
     gp_model <- GPModel(group_data = group)
     opt <- optim(par=c(1,1), fn=gp_model$neg_log_likelihood, y=y, method="Nelder-Mead")
-    expect_lt(sum(abs(opt$par-cov_pars[c(1,3)])),TOLERANCE)
+    expect_lt(sum(abs(opt$par-cov_pars[c(1,3)])),TOLERANCE_MEDIUM)
     expect_lt(abs(opt$value-(1228.293)),1E-2)
     expect_equal(as.integer(opt$counts[1]), 49)
     
@@ -172,13 +174,38 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     gp_model <- GPModel(group_data = group[shuffle_ind])
     fit(gp_model, y = y[shuffle_ind], params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE,
                                                     convergence_criterion = "relative_change_in_parameters"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
     
     # Inf in response variable data gives error
     y_Inf <- y
     y_Inf[1] = -Inf
     expect_error(gp_model <- fitGPModel(group_data = group, y = y_Inf))
+    
+    # With offset / fixed_effects
+    offset <- 20 * sim_rand_unif(n=n, init_c=0.354)
+    y_o <- y + offset
+    cov_pars_pred = c(0.5,1.5)
+    nrounds <- 6
+    cov_pars <- c(0.49348233, 0.02326298, 1.22299112, 0.17995099)
+    expected_mu <- c(-0.1553877, -0.3945731, 0)
+    expected_cov <- c(0.5483871, 0.0000000, 0.0000000, 0.0000000,
+                      0.5483871, 0.0000000, 0.0000000, 0.0000000, 2)
+    gp_model <- fitGPModel(group_data = group,
+                           y = y,  params = DEFAULT_OPTIM_PARAMS_STD)
+    pred <- predict(gp_model, group_data_pred = group_test,
+                    cov_pars = cov_pars_pred, predict_cov_mat = TRUE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
+    expect_equal(gp_model$get_num_optim_iter(), nrounds)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
+    gp_model <- fitGPModel(group_data = group, y = y_o, fixed_effects = offset,
+                           params = DEFAULT_OPTIM_PARAMS_STD)
+    pred <- predict(gp_model,group_data_pred = group_test, fixed_effects = offset,
+                    cov_pars = cov_pars_pred, predict_cov_mat = TRUE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
+    expect_equal(gp_model$get_num_optim_iter(), nrounds)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
   })
   
   test_that("linear mixed effects model with grouped random effects ", {
@@ -197,9 +224,11 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          convergence_criterion = "relative_change_in_parameters"))
     cov_pars <- c(0.49205230, 0.02319557, 1.22064076, 0.17959832)
     coef <- c(2.07499902, 0.11269252, 1.94766255, 0.03382472)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 7)
+    nll <- 1226.885947
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_STRICT)
+    expect_lt(abs(gp_model$get_current_neg_log_likelihood() - nll), TOLERANCE_STRICT)
+    expect_equal(gp_model$get_num_optim_iter(), 6)
     
     # Prediction
     group_test <- c(1,2,m+1)
@@ -210,8 +239,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.886494, 2.043259, 2.854064)
     expected_cov <- c(0.5393509 , 0.0000000000, 0.0000000000, 0.0000000000,
                       0.5393509 , 0.0000000000, 0.0000000000, 0.0000000000, 1.712693)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
     
     # Predict training data random effects
     all_training_data_random_effects <- predict_training_data_random_effects(gp_model)
@@ -220,41 +249,70 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_unique <- unique(group)
     X_zero <- cbind(rep(0,length(group_unique)),rep(0,length(group_unique)))
     pred_random_effects <- predict(gp_model, group_data_pred = group_unique, X_pred = X_zero)
-    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOL_STRICT)
+    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOLERANCE_STRICT)
     
     # Fit model using gradient descent instead of wls for regression coefficients
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "gradient_descent", maxit=1000, std_dev = TRUE,
                                          optimizer_coef = "gradient_descent", lr_coef=1, use_nesterov_acc=TRUE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_LOOSE)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_LOOSE)
-    expect_equal(gp_model$get_num_optim_iter(), 9)
-    
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_LOOSE)
+    expect_lt(abs(gp_model$get_current_neg_log_likelihood() - nll), TOLERANCE_LOOSE)
     # Fit model using Nelder-Mead
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "nelder_mead",
                                          optimizer_coef = "nelder_mead", std_dev = FALSE, delta_rel_conv=1e-6))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOL_LOOSE)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef[c(1,3)])),TOL_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOLERANCE_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef[c(1,3)])),TOLERANCE_LOOSE)
     expect_equal(gp_model$get_num_optim_iter(), 125)
+    expect_lt(abs(gp_model$get_current_neg_log_likelihood() - nll), TOLERANCE_LOOSE)
     # Fit model using Adam
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "adam", std_dev = FALSE, delta_rel_conv=1e-6))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOL_LOOSE)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef[c(1,3)])),TOL_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOLERANCE_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef[c(1,3)])),TOLERANCE_LOOSE)
     expect_equal(gp_model$get_num_optim_iter(), 354)
     
-    # "bfgs" produces large discrepancies -> not run (15.11.2023)
-    # # Fit model using BFGS
-    # gp_model <- fitGPModel(group_data = group,
-    #                        y = y, X = X,
-    #                        params = list(optimizer_cov = "bfgs", std_dev = TRUE))
-    # expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
-    # expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
-    # expect_equal(gp_model$get_num_optim_iter(), 2)
+    # Fit model using lbfgs
+    gp_model <- fitGPModel(group_data = group,
+                           y = y, X = X,
+                           params = list(optimizer_cov = "lbfgs", std_dev = TRUE))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_LOOSE)
+    expect_lt(abs(gp_model$get_current_neg_log_likelihood() - nll), TOLERANCE_LOOSE)
+    expect_equal(gp_model$get_num_optim_iter(), 8)
+    
+    # With offset / fixed_effects
+    offset <- 20 * sim_rand_unif(n=n, init_c=0.1354)
+    y_o <- y + offset
+    group_test <- c(1,2,m+1)
+    X_test <- cbind(rep(1,3),c(-0.5,0.2,0.4))
+    cov_pars <- c(0.49242554365, 0.02321316274, 1.21953176715, 0.17944680016)
+    coef <- c(2.07503748836, 0.11264496521, 1.94644611839, 0.03383742417)
+    nrounds <- 7
+    expected_mu <- c(0.887391, 2.044038, 2.853616)
+    expected_cov <- c(0.5397569, 0.0000000, 0.0000000, 0.0000000, 0.5397569, 0.0000000, 0.0000000, 0.0000000, 1.7119573)
+    gp_model <- fitGPModel(group_data = group,
+                           y = y, X = X, params = DEFAULT_OPTIM_PARAMS_STD)
+    pred <- predict(gp_model, group_data_pred = group_test,
+                    X_pred = X_test, predict_cov_mat = TRUE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_STRICT)
+    expect_equal(gp_model$get_num_optim_iter(), nrounds)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
+    gp_model <- fitGPModel(group_data = group, y = y_o, X = X, 
+                           fixed_effects = offset, params = DEFAULT_OPTIM_PARAMS_STD)
+    pred <- predict(gp_model, group_data_pred = group_test, fixed_effects = offset,
+                    X_pred = X_test, predict_cov_mat = TRUE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_STRICT)
+    expect_equal(gp_model$get_num_optim_iter(), nrounds)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
     
     # Large data
     n_L <- 1e6 # number of samples
@@ -269,23 +327,20 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     y_L <- b1_L[group_L] + X_L%*%beta + xi_L
     # Fit model using gradient descent instead of wls for regression coefficients
     gp_model <- fitGPModel(group_data = group_L,
-                           y = y_L, X = X_L,
-                           params = list(optimizer_cov = "gradient_descent", maxit=1000, std_dev = TRUE,
-                                         optimizer_coef = "gradient_descent", lr_coef=0.1, use_nesterov_acc=TRUE))
-    cov_pars <- c(0.500507115, 0.000746112, 0.998467820, 0.004689711)
-    coef <- c(1.995528216, 0.003485014, 2.001517023, 0.002577151)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 7)
-    expect_lt(sum(abs(gp_model$get_current_neg_log_likelihood() - 1224957.3892409)),TOL_STRICT)
+                           y = y_L, X = X_L, params = DEFAULT_OPTIM_PARAMS_STD)
+    cov_pars <- c(0.5004967636668, 0.0007460965241, 0.9983817688906, 0.0046893215444)
+    coef <- c(1.995473876130, 0.003484883983, 2.001625713274, 0.002577124175)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(gp_model$get_current_neg_log_likelihood() - 1224957.387)),TOLERANCE_MEDIUM)
     
     gp_model <- fitGPModel(group_data = group_L, y = y_L, X = X_L,
                            params = list(optimizer_cov = "nelder_mead", maxit=1000, delta_rel_conv=1e-6))
     cov_pars <- c(0.5004681, 0.9988288)
     coef <- c(2.000747, 1.999343)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
-    expect_lt(sum(abs(gp_model$get_current_neg_log_likelihood() - 1224958.6057028)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(gp_model$get_current_neg_log_likelihood() - 1224958.6057028)),TOLERANCE_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 152)
     
   })
@@ -298,8 +353,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     gp_model <- fitGPModel(group_data = cbind(group,group2), y = y, 
                            params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE))
     expected_values <- c(0.49792062, 0.02408196, 1.21972166, 0.18357646, 1.06962710, 0.22567292)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 5)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOLERANCE_MEDIUM)
+    expect_equal(gp_model$get_num_optim_iter(), 4)
     
     # Predict training data random effects
     cov_pars <- gp_model$get_cov_pars()[1,]
@@ -313,21 +368,21 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_data_pred = cbind(group_unique,rep(-1,length(group_unique)))
     preds <- predict(gp_model, group_data_pred=group_data_pred,
                      predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_effects[,1] - preds$mu)),TOL_STRICT)
-    expect_lt(sum(abs(pred_random_effects[,2] - (preds$var - cov_pars[3]))),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_effects[,1] - preds$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(pred_random_effects[,2] - (preds$var - cov_pars[3]))),TOLERANCE_STRICT)
     # Check whether crossed random effects are correct
     group_unique <- unique(group2)
     group_data_pred = cbind(rep(-1,length(group_unique)),group_unique)
     preds <- predict(gp_model, group_data_pred=group_data_pred,
                      predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_effects_crossed[,1] - preds$mu)),TOL_STRICT)
-    expect_lt(sum(abs(pred_random_effects_crossed[,2] - (preds$var - cov_pars[2]))),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_effects_crossed[,1] - preds$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(pred_random_effects_crossed[,2] - (preds$var - cov_pars[2]))),TOLERANCE_STRICT)
     # Prediction after training
     group_data_pred = cbind(c(1,1,m+1),c(2,1,length(group2)+1))
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred, predict_var = TRUE)
     expected_mu <- c(0.7509175, -0.4208015, 0.0000000)
     expected_var <- c(0.5677178, 0.5677178, 2.7872694)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
     expect_lt(sum(abs(as.vector(pred$var)-expected_var)),1E-4)
     # Prediction without training and parameters given
     gp_model <- GPModel(group_data = cbind(group,group2))
@@ -336,24 +391,24 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.7631462, -0.4328551, 0.000000000)
     expected_cov <- c(0.114393721, 0.009406189, 0.0000000, 0.009406189,
                       0.114393721 , 0.0000000, 0.0000000, 0.0000000, 3.100000000)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_MEDIUM)
     # Prediction for only existing random effects
     group_data_pred_in = cbind(c(1,1),c(2,1))
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred_in,
                              cov_pars = c(0.1,1,2), predict_cov_mat = TRUE)
     expected_mu <- c(0.7631462, -0.4328551)
     expected_cov <- c(0.114393721, 0.009406189, 0.009406189, 0.114393721)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_MEDIUM)
     # Prediction for only new random effects
     group_data_pred_out = cbind(c(m+1,m+1,m+1),c(length(group2)+1,length(group2)+2,length(group2)+1))
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred_out,
                              cov_pars = c(0.1,1,2), predict_cov_mat = TRUE)
     expected_mu <- c(rep(0,3))
     expected_cov <- c(3.1, 1.0, 3.0, 1.0, 3.1, 1.0, 3.0, 1.0, 3.1)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_MEDIUM)
     
     ## Two crossed random effects and a random slope
     y <- Z1%*%b1 + Z2%*%b2 + Z3%*%b3 + xi
@@ -363,7 +418,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "fisher_scoring", maxit=5, std_dev = TRUE))
     expected_values <- c(0.49554952, 0.02546769, 1.24880860, 0.18983953, 1.05505134, 0.22337199, 1.13840014, 0.17950490)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOLERANCE_MEDIUM)
     expect_equal(gp_model$get_num_optim_iter(), 5)
     
     # Predict training data random effects
@@ -379,21 +434,21 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     x_pr = rep(0,length(group_unique))
     preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr, 
                      predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_effects[,1] - preds$mu)),TOL_STRICT)
-    expect_lt(sum(abs(pred_random_effects[,2] - (preds$var - cov_pars[3]))),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_effects[,1] - preds$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(pred_random_effects[,2] - (preds$var - cov_pars[3]))),TOLERANCE_STRICT)
     # Check whether random slopes are correct
     x_pr = rep(1,length(group_unique))
     preds2 <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr,
                       predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_slopes[,1] - (preds2$mu-preds$mu))),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_slopes[,1] - (preds2$mu-preds$mu))),TOLERANCE_STRICT)
     # Check whether crossed random effects are correct
     group_unique <- unique(group2)
     group_data_pred = cbind(rep(-1,length(group_unique)),group_unique)
     x_pr = rep(0,length(group_unique))
     preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr,
                      predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_effects_crossed[,1] - preds$mu)),TOL_STRICT)
-    expect_lt(sum(abs(pred_random_effects_crossed[,2] - (preds$var - cov_pars[2]))),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_effects_crossed[,1] - preds$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(pred_random_effects_crossed[,2] - (preds$var - cov_pars[2]))),TOLERANCE_STRICT)
     
     # Prediction
     gp_model <- GPModel(group_data = cbind(group,group2),
@@ -408,14 +463,14 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.7579961, -0.2868530, 0.000000000)
     expected_cov <- c(0.11534086, -0.01988167, 0.0000000, -0.01988167, 2.4073302,
                       0.0000000, 0.0000000, 0.0000000, 3.235)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_MEDIUM)
     # Predict variances
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
                              group_rand_coef_data_pred=group_rand_coef_data_pred,
                              cov_pars = c(0.1,1,2,1.5), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE_MEDIUM)
     
     # Gradient descent
     gp_model <- fitGPModel(group_data = cbind(group,group2),
@@ -424,7 +479,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE))
     cov_pars <- c(0.4958303, 1.2493181, 1.0543343, 1.1388604 )
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_STRICT)
     expect_equal(gp_model$get_num_optim_iter(),8)
     
     # Nelder-Mead
@@ -434,20 +489,23 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "nelder_mead", std_dev = FALSE, delta_rel_conv=1e-6))
     expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_VERY_LOOSE)
-    # BFGS
-    gp_model <- fitGPModel(group_data = cbind(group,group2),
-                           group_rand_coef_data = x,
-                           ind_effect_group_rand_coef = 1,
-                           y = y,
-                           params = list(optimizer_cov = "bfgs", std_dev = FALSE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_LOOSE)
+    # lbfgs
+    gp_model <- fitGPModel(group_data = cbind(group,group2),  group_rand_coef_data = x,
+                           ind_effect_group_rand_coef = 1, y = y,
+                           params = list(optimizer_cov = "lbfgs", std_dev = FALSE))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
+    # bfgs_optim_lib
+    gp_model <- fitGPModel(group_data = cbind(group,group2),  group_rand_coef_data = x,
+                           ind_effect_group_rand_coef = 1, y = y,
+                           params = list(optimizer_cov = "bfgs_optim_lib", std_dev = FALSE))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
     # Adam
     gp_model <- fitGPModel(group_data = cbind(group,group2),
                            group_rand_coef_data = x,
                            ind_effect_group_rand_coef = 1,
                            y = y,
                            params = list(optimizer_cov = "adam", std_dev = FALSE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_LOOSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE_LOOSE)
     
     # Evaluate negative log-likelihood
     nll <- gp_model$neg_log_likelihood(cov_pars=c(0.1,1,2,1.5),y=y)
@@ -463,7 +521,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            drop_intercept_group_rand_effect = c(TRUE,FALSE), y = y,
                            params = list(optimizer_cov = "gradient_descent"))
     expected_values <- c(0.5017205, 1.0818474, 1.1157430)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOLERANCE_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 7)
     
     # Predict training data random effects
@@ -479,16 +537,16 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     x_pr = rep(1,length(group_unique))
     preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr,
                      predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_slopes[,1] - preds$mu)),TOL_STRICT)
-    expect_lt(sum(abs(pred_random_slopes[,2] - (preds$var - cov_pars[2]))),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_slopes[,1] - preds$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(pred_random_slopes[,2] - (preds$var - cov_pars[2]))),TOLERANCE_STRICT)
     # Check whether crossed random effects are correct
     group_unique <- unique(group2)
     group_data_pred = cbind(rep(-1,length(group_unique)),group_unique)
     x_pr = rep(0,length(group_unique))
     preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr,
                      predict_var = TRUE, predict_response = FALSE)
-    expect_lt(sum(abs(pred_random_effects_crossed[,1] - preds$mu)),TOL_STRICT)
-    expect_lt(sum(abs(pred_random_effects_crossed[,2] - preds$var)),TOL_STRICT)
+    expect_lt(sum(abs(pred_random_effects_crossed[,1] - preds$mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(pred_random_effects_crossed[,2] - preds$var)),TOLERANCE_STRICT)
     
     # Prediction
     gp_model <- GPModel(group_data = cbind(group,group2),
@@ -502,14 +560,14 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.8426751, -0.5964363, 0.000000000)
     expected_cov <- c(0.10558205, -0.01269261, 0.00000000, -0.01269261, 2.40180871,
                       0.00000000, 0.00000000, 0.00000000, 2.23500000)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_MEDIUM)
     # Predict variances
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
                              group_rand_coef_data_pred=group_rand_coef_data_pred,
                              cov_pars = c(0.1,2,1.5), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE_MEDIUM)
     
   })
   
@@ -522,8 +580,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          convergence_criterion = "relative_change_in_parameters"))
                     , file='NUL')
     expected_values <- c(0.49348532, 0.02326312, 1.22299521, 0.17995161)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOLERANCE_STRICT)
     
     # gradient descent
     capture.output( gp_model <- fitGPModel(group_data = group, cluster_ids = cluster_ids,
@@ -533,8 +590,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          convergence_criterion = "relative_change_in_parameters"))
                     , file='NUL')
     cov_pars_expected <- c(0.49348532, 0.02326312, 1.22299520, 0.17995161)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 8)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),TOLERANCE_STRICT)
     
     # Predict training data random effects
     all_training_data_random_effects <- predict_training_data_random_effects(gp_model)
@@ -543,7 +599,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_unique <- unique(group)
     pred_random_effects <- predict(gp_model, group_data_pred = group_unique, 
                                    cluster_ids_pred = cluster_ids[first_occurences])
-    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOL_STRICT)
+    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOLERANCE_STRICT)
     
     # Prediction
     group_data_pred = c(1,1,m+1)
@@ -557,8 +613,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(-0.1514786, 0.000000, 0.000000)
     expected_cov <- c(0.8207547, 0.000000, 0.000000, 0.000000,
                       2.000000, 0.000000, 0.000000, 0.000000, 2.000000)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
     
     # cluster_ids can be string 
     cluster_ids_string <- paste0(as.character(cluster_ids),"_s")
@@ -567,16 +623,15 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "fisher_scoring", maxit=100, std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_parameters"))
                     , file='NUL')
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),TOL_STRICT)
-    expect_equal(gp_model$get_num_optim_iter(), 6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),TOLERANCE_STRICT)
     # Prediction
     group_data_pred = c(1,1,m+1)
     cluster_ids_pred_string = paste0(as.character(c(1,3,1)),"_s")
     pred <- gp_model$predict(y = y, group_data_pred = group_data_pred,
                              cluster_ids_pred = cluster_ids_pred_string,
                              cov_pars = c(0.75,1.25), predict_cov_mat = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE_STRICT)
     
     # Prediction when cluster_ids are not in ascending order
     group_data_pred = c(1,1,m,m)
@@ -587,8 +642,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                              cov_pars = c(0.75,1.25), predict_var = TRUE)
     expected_mu <- c(rep(0,3), 1.179557)
     expected_var <- c(rep(2,3), 0.8207547)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),TOLERANCE_STRICT)
     # same thing when cluster_ids are strings
     group_data_pred = c(1,1,m,m)
     cluster_ids_pred_string = paste0(as.character(c(2,2,1,2)),"_s")
@@ -596,8 +651,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     pred <- gp_model$predict(y = y, group_data_pred = group_data_pred,
                              cluster_ids_pred = cluster_ids_pred_string,
                              cov_pars = c(0.75,1.25), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),TOL_STRICT)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOLERANCE_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),TOLERANCE_STRICT)
 
     # cluster_ids and random coefficients
     y <- Z1%*%b1 + Z3%*%b3 + xi
@@ -611,7 +666,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          convergence_criterion = "relative_change_in_parameters"))
                     , file='NUL')
     expected_values <- c(0.4927786, 1.2565102, 1.1333662)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOLERANCE_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 13)
     
   })
